/* Copyright 2015, Michele Santullo
 * This file is part of DoorKeeper.
 *
 * DoorKeeper is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * DoorKeeper is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with DoorKeeper.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef id372857FBD20C4DD2A9EDDB3A167682AA
#define id372857FBD20C4DD2A9EDDB3A167682AA

#include "doorkeeper/implem/maptypes.hpp"
#include "doorkeeper/primitivetypes.hpp"
#include "doorkeeper/helpers/typename.hpp"
#include "doorkeeper/implem/coords_utils.hpp"
#include <cstdint>
#include <vector>
#include <string>
#include <type_traits>
#include <cstddef>

namespace dk {
	template <uint32_t D>
	class PixelConv;

	namespace implem {
		template <typename T, uint32_t D, bool POD=std::is_pod<T>::value>
		class data_fetcher;
	} //namespace implem

	template <uint32_t D>
	class BaseMapSource {
		template <typename T, uint32_t, bool POD>
		friend class implem::data_fetcher;
	public:
		typedef dk::Vector<D> coords;
		BaseMapSource ( void ) = default;
		virtual ~BaseMapSource ( void ) noexcept = default;

		template <typename T>
		void fetch ( std::vector<T>& parOut, const coords& parFrom, const coords& parTo );

		virtual const coords& mapSize ( void ) const = 0;
		virtual const coords& tileSize ( void ) const = 0;
		virtual MapTypes mapType ( void ) const = 0;
		virtual int layersCount ( void ) const = 0;
		virtual void chainedMaps ( std::vector<std::string>& parOut ) const = 0;
		virtual HashType layerTypeHash ( int parIndex ) const = 0;
		virtual const PixelConv<D>& pixel_conv ( void ) const = 0;

	protected:
		virtual void fetch_raw ( char* parOut, const coords& parFrom, const coords& parTo, std::size_t parSize ) = 0;
	};
} //namespace dk

#include "doorkeeper/implem/basemapsource.inl"

#endif
