/* Copyright 2015, Michele Santullo
 * This file is part of DoorKeeper.
 *
 * DoorKeeper is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * DoorKeeper is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with DoorKeeper.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef idD3EDC396AA314474B3D909559FFC0247
#define idD3EDC396AA314474B3D909559FFC0247

#include "doorkeeper/primitivetypes.hpp"
#include "doorkeeper/components/tileiterator.hpp"
#include "doorkeeper/components/basemapsource.hpp"
#include "doorkeeper/implem/helpers.hpp"
#include <algorithm>
#include <vector>
#include <memory>
#include <cstdint>
#if !defined(NDEBUG)
#include <iostream>
#endif
#include <functional>

namespace dk {
	template <uint32_t D>
	class Viewport;

	template <uint32_t D>
	class LayerBase {
	public:
		typedef Vector<D> coords;

		LayerBase ( const coords& parCount, const coords& parTileSize );
		virtual ~LayerBase ( void ) noexcept = default;

		void preload ( const coords& parFrom, const coords& parTo );
		const coords& map_size ( void ) const { return m_count; }
		const coords& tile_size ( void ) const { return m_tilesize; }

	protected:
		coords m_count;
		coords m_tilesize;

	private:
		virtual void onPreload ( const coords& parFrom, const coords& parTo ) = 0;

		coords m_haveFrom;
		coords m_haveTo;
	};

	template <typename T, uint32_t D>
	class Layer : public LayerBase<D> {
		friend class Viewport<D>;

	public:
		typedef typename LayerBase<D>::coords coords;
		typedef TileIterator<T, D> iterator;
		typedef TileIterator<const T, D> const_iterator;

		Layer ( const Layer& ) = delete;
		Layer ( Layer&& ) = default;
		explicit Layer ( BaseMapSource<D>* parTilemap );
		virtual ~Layer ( void ) noexcept = default;

		Layer& operator= ( const Layer& ) = delete;

		iterator begin ( void );
		iterator end ( void );

	private:
		virtual void onPreload ( const coords& parFrom, const coords& parTo );

		std::vector<T> m_tiles;
		BaseMapSource<D>* m_tilemap;
	};

	namespace implem {
		template <uint32_t D, typename O>
		bool at_least_one ( const Vector<D>& parLeft, const Vector<D>& parRight, O parOp ) a_pure;
	} //namespace implem
} //namespace dk

#include "doorkeeper/implem/layer.inl"

#endif
