/* Copyright 2015, Michele Santullo
 * This file is part of DoorKeeper.
 *
 * DoorKeeper is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * DoorKeeper is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with DoorKeeper.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef id83407244096746CDBFF64D23F0B1701A
#define id83407244096746CDBFF64D23F0B1701A

#include "doorkeeper/primitivetypes.hpp"
#include "doorkeeper/components/viewport.hpp"
#include "doorkeeper/components/tyler.hpp"
#include "doorkeeper/components/layer.hpp"
#include <map>
#include <boost/iterator/transform_iterator.hpp>
#include <utility>
#include <functional>

namespace dk {
	template <uint32_t D>
	class Tyler;

	template <uint32_t D>
	class LayeredViewport {
		typedef std::map<TileInfo<D>, Viewport<D>> ViewportList;
	public:
		typedef VectorT<CoordinateAccumType, D> coords_f;
		typedef boost::transform_iterator<std::function<Viewport<D>&(const LayerBase<D>&)>, typename Tyler<D>::iterator> iterator;
		typedef boost::transform_iterator<std::function<const Viewport<D>&(const LayerBase<D>&)>, typename Tyler<D>::const_iterator> const_iterator;

		explicit LayeredViewport ( Tyler<D>& parTyler );
		~LayeredViewport ( void ) noexcept;

		LayeredViewport<D>& operator+= ( const coords_f& parValue );
		LayeredViewport<D>& operator-= ( const coords_f& parValue );

		void on_layer_added ( const LayerBase<D>* parLayer );
		void on_layer_removed ( const LayerBase<D>* parLayer );
		iterator begin ( void );
		const_iterator begin ( void ) const;
		iterator end ( void );
		const_iterator end ( void ) const;
		Viewport<D>& viewport ( const LayerBase<D>& parLayer );
		const Viewport<D>& viewport ( const LayerBase<D>& parLayer ) const;

	private:
		ViewportList m_views;
		Tyler<D>& m_tyler;
	};
} //namespace dk

#include "doorkeeper/implem/layeredviewport.inl"

#endif
