/* Copyright 2015, Michele Santullo
 * This file is part of DoorKeeper.
 *
 * DoorKeeper is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * DoorKeeper is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with DoorKeeper.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef idA52FEA0859494D3FBDF8ED5565091C59
#define idA52FEA0859494D3FBDF8ED5565091C59

#include "doorkeeper/components/tyler.hpp"
#include "doorkeeper/components/basemapsource.hpp"
#include "doorkeeper/helpers/typename.hpp"
#include "doorkeeper/components/exception.hpp"
#include <cstdint>
#include <utility>
#include <string>
#include <memory>
#include <map>
#include <stack>
#include <ciso646>

namespace dkh {
	template <uint32_t D>
	using PushLayerMapType = std::map<dk::HashType, void(dk::Tyler<D>::*)(dk::BaseMapSource<D>*,int)>;

	template <uint32_t D, typename C>
	struct MapLoaderPool {
		typedef std::unique_ptr<dk::BaseMapSource<D>> BaseMapSourceUPtr;
		typedef std::map<std::string, BaseMapSourceUPtr> PoolMapType;

		enum { dimensions = D };
		typedef C opener_type;

		PoolMapType pool;
		C opener;

		dk::BaseMapSource<D>* operator() ( const std::string& parName );
	};

	class UnknownLayerTemplateException : public dk::DoorKeeperException {
	};

	template <typename M>
	dk::Tyler<M::dimensions> call_map_load ( M& parFileOpener, const std::string& parOpen, const PushLayerMapType<M::dimensions>& parPusher );

	template <typename M>
	dk::Tyler<M::dimensions> call_map_load ( dk::Tyler<M::dimensions>& parTyler, M& parFileOpener, const std::string& parOpen, const PushLayerMapType<M::dimensions>& parPusher );

	template <uint32_t D, typename C>
	dk::Tyler<D> map_load ( C& parFileOpener, const std::string& parOpen, const PushLayerMapType<D>& parPusher );

	template <uint32_t D, typename C>
	dk::Tyler<D>& map_load ( dk::Tyler<D>& parTyler, C& parFileOpener, const std::string& parOpen, const PushLayerMapType<D>& parPusher );

	namespace implem {
		template <uint32_t D>
		void call_push_layer ( dk::Tyler<D>& parTyler, const PushLayerMapType<D>& parPusher, dk::BaseMapSource<D>* parReader, int parIndex );
	} //namespace implem
} //namespace dkh

#include "doorkeeper/implem/maploader.inl"

#endif
