/* Copyright 2015, Michele Santullo
 * This file is part of DoorKeeper.
 *
 * DoorKeeper is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * DoorKeeper is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with DoorKeeper.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace dk {
	template <typename T, uint32_t D, typename T1>
	TileIterator<T, D, T1>::TileIterator (qualif_vector_type* parData, const PixelConv<D>& parPixConv, const coords& parArea) :
		m_tile_range(parArea),
		m_data(parData),
		m_pixel_conv(parPixConv)
	{
		DK_ASSERT(parData);
		DK_ASSERT(&m_pixel_conv != nullptr);
	}

	template <typename T, uint32_t D, typename T1>
	TileIterator<T, D, T1>::TileIterator (qualif_vector_type* parData, const PixelConv<D>& parPixConv, const coords& parStart, const coords& parArea) :
		m_tile_range(parStart, parArea),
		m_data(parData),
		m_pixel_conv(parPixConv)
	{
		DK_ASSERT(parData);
		DK_ASSERT(&m_pixel_conv != nullptr);
	}

	template <typename T, uint32_t D, typename T1>
	void TileIterator<T, D, T1>::increment() {
		++m_tile_range;
	}

	template <typename T, uint32_t D, typename T1>
	void TileIterator<T, D, T1>::decrement() {
		--m_tile_range;
	}

	template <typename T, uint32_t D, typename T1>
	void TileIterator<T, D, T1>::advance (size_t parAdvance) {
		constexpr auto M = std::numeric_limits<CoordinateScalarType>::max();
		typedef typename std::conditional<
			(static_cast<std::make_unsigned<size_t>::type>(std::numeric_limits<size_t>::max()) > static_cast<std::make_unsigned<CoordinateScalarType>::type>(M)),
			size_t,
			CoordinateScalarType
		>::type LargestType;

		while (parAdvance) {
			const auto inc = std::min(static_cast<LargestType>(parAdvance), static_cast<LargestType>(M));
			m_tile_range += inc;
			parAdvance -= static_cast<size_t>(inc);
		}
	}

	template <typename T, uint32_t D, typename T1>
	auto TileIterator<T, D, T1>::distance_to (const TileIterator& parOther) -> difference_type {
		return std::distance(to_index(m_tile_range), to_index(parOther.m_tile_range));
	}

	template <typename T, uint32_t D, typename T1>
	bool TileIterator<T, D, T1>::equal (const TileIterator& parOther) const {
		return m_data == parOther.m_data and m_tile_range.position() == parOther.m_tile_range.position();
	}

	template <typename T, uint32_t D, typename T1>
	auto TileIterator<T, D, T1>::dereference() const -> reference {
		const auto index = to_index(m_tile_range);
		DK_ASSERT(m_data);
		DK_ASSERT(index >= 0);
		DK_ASSERT(static_cast<typename std::make_unsigned<decltype(index)>::type>(index) < m_data->size());
		return Tile<T, D>(&(*m_data)[index], m_tile_range, coords(0), &m_pixel_conv);
	}

	template <uint32_t D>
	inline Vector<D> make_past_end_coords (const Vector<D>& parTo) {
		Vector<D> retval(0);
		//for (CoordinateDistType d = 0; d < D - 1; ++d) {
			//retval[d] = parFrom[d];
		//}
		retval[D - 1] = parTo[D - 1] + 1;
		return retval;
	}
} //namespace dk
